<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://www.phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/admin_category.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

class CalCategory extends CalServer {
    private static $instance = NULL;
    private $cat_data = [];
    private $calcat = [
        'fh_cat_id'    => '',
        'fh_cat_name'  => '',
        'fh_cat_color' => ''
    ];

    public function __construct() {
        parent::__construct();
    }

    public static function getInstance() {
        if (self::$instance == NULL) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function startAdmin() {
        if (check_get('cancel')) {
            redirect(FUSION_SELF.fusion_get_aidlink());
        }

        if (check_get('ref') && get('ref') == "categoryform") {
            $this->categoryForm();
        } else {
        	$this->categoryQuery();
        }
    }

    private function categoryForm() {
        if (check_get('action') && (get('action') == "edit") && (check_get('cat_id', FILTER_VALIDATE_INT))) {
            $result = dbquery($this->sqlCal->getCategoryQuery(['condition' => "fh_cat_id = ".(int)get('cat_id').""]));
            if (dbrows($result)) {
                $this->cat_data = dbarray($result);
            } else {
                redirect(FUSION_SELF.fusion_get_aidlink());
            }
        }
        if (check_get('action') && (get('action') == "delete") && check_get('cat_id', FILTER_VALIDATE_INT)) {
            $cat_id = get('cat_id');

            if (dbcount("(fh_cat_id)", DB_FHCAL_CATEGORY, "fh_cat_id = :catid", [':catid' => $cat_id])) {
                dbquery("DELETE FROM ".DB_FHCAL_CATEGORY." WHERE fh_cat_id=:catid", [':catid' => $cat_id]);
                addnotice('success', self::$locale['FCAL_128']);
            }

            redirect(clean_request('', ['ref', 'action', 'cat_id'], FALSE));
        }
        $this->cat_data += $this->calcat;
        $this->contentForm();
    }

    private function categoryQuery() {
            $result = dbquery($this->sqlCal->getCategoryQuery());
            if (dbrows($result)) {
                while ($data = dbarray($result)) {
                    $this->cat_data[] = $data;
                }
            } else {
            	$this->cat_data = [];
            }
            $this->categoryListing();
    }

    private function contentForm() {
        if ( check_post( 'savesettings' ) ) {
            $cat_data = [
                'fh_cat_id'    => sanitizer( 'fh_cat_id', '', 'fh_cat_id' ),
                'fh_cat_name'  => sanitizer( 'fh_cat_name', '', 'fh_cat_name' ),
                'fh_cat_color' => sanitizer( 'fh_cat_color', '', 'fh_cat_color' )
            ];

            if ( fusion_safe() ) {
                $dataid = dbquery_insert( DB_FHCAL_CATEGORY, $cat_data, empty( $cat_data['fh_cat_id'] ) ? 'save' : 'update' );
                addnotice( 'success', empty( $cat_data['fh_cat_id'] ) ? self::$locale['FCAL_129'] : self::$locale['FCAL_130'] );
                redirect( clean_request( '', [ 'cat_id', 'action', 'ref' ], FALSE ) );
            }
        }

        echo openform( 'settingsFrm', 'POST' );
        echo form_hidden('fh_cat_id', '', $this->cat_data['fh_cat_id']);
        echo form_text( 'fh_cat_name', self::$locale['FCAL_131'], $this->cat_data['fh_cat_name'], [
            'inline'     => TRUE,
            'required'   => TRUE,
            'max_length' => 255
        ] );
        echo form_colorpicker( 'fh_cat_color', self::$locale['FCAL_132'], $this->cat_data['fh_cat_color'], [
            'inline'     => TRUE
        ] );
        echo form_button( 'savesettings', self::$locale['submit'], self::$locale['submit'], [ 'class' => 'btn-primary' ] );
        echo closeform();
    }

    private function categoryListing() {
        echo openform('calendar_filter', 'post', FUSION_REQUEST);
        echo "<div class='m-b-5'>";
        echo "<div class='clearfix'>";
            echo "<div class='text-right'>";
                echo "<a class='btn btn-success btn-sm' href=" . clean_request( "ref=categoryform", ["ref"], FALSE ) . "><i class='fa fa-fw fa-plus'></i>" . self::$locale['FCAL_133'] . "</a>";
            echo "</div>";
        echo closeform();
        if (!empty($this->cat_data)) {
            echo "<div class='table-responsive'>";
                echo "<table class='table table-striped'>";
                    echo "<thead>";
                    echo "<tr>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_131'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_132'] . "</th>";
                        echo "<th class='text-center'></th>";
                    echo "</tr>";
                    echo "</thead>";
                    echo "<tbody>\n";
                    foreach ( $this->cat_data as $cdata ) {
                        $edit_link = clean_request("section=category&ref=categoryform&action=edit&cat_id=".$cdata['fh_cat_id'], ['section', 'ref', 'action', 'fh_cat_id'], FALSE);
                        $delete_link = clean_request("section=category&ref=categoryform&action=delete&cat_id=".$cdata['fh_cat_id'], ['section', 'ref', 'action', 'fh_cat_id'], FALSE);

                        echo "<tr>";
                            echo "<td class='text-center'>" . $cdata['fh_cat_name'] . "</td>\n";
                            echo "<td class='text-left'><i class='fa fa-square fa-2x' style='color:" . $cdata['fh_cat_color'] . ";'></i></td>\n";
                            echo "<td class='text-center'><a href='".$edit_link."'><i class='fa fa-edit fa-fw'></i></a>" . ($cdata['fh_cat_id'] > 4 ? " <a href='".$delete_link."'><i class='fa fa-trash fa-fw'></i></a>" : '') . "</td>\n";
                        echo "</tr>";
                    }
                    echo "</tbody>";
                echo "</table>";
            echo "</div>";

        } else {
            echo "<div class='well text-center'>nincs adat</div>";
        }
        echo "</div>";
        echo "</div>";
    }

}
