<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://www.phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/admin_settings.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

class CalHoliDays extends CalServer {
    private static $instance = NULL;
    private $holiday_data = [];
    private $calholiday = [
        'fh_hday_id'   => '',
        'fh_hday_name' => '',
        'fh_hday_date' => ''
    ];

    public function __construct() {
        parent::__construct();
    }

    public static function getInstance() {
        if (self::$instance == NULL) {
            self::$instance = new static();
        }

        return self::$instance;
    }

    public function startAdmin() {
        if (check_get('ref') && get('ref') == "holidaysform") {
            $this->holidaysForm();
        } else {
        	$this->holidaysQuery();
        }
    }

    private function holidaysForm() {
        if (check_get('action') && (get('action') == "edit") && (check_get('hday_id', FILTER_VALIDATE_INT))) {
            $result = dbquery($this->sqlCal->getHolidayQuery(['condition' => "fh_hday_id = " . (int)get('hday_id') . ""]));
            if (dbrows($result)) {
                $this->holiday_data = dbarray($result);
            } else {
                redirect(FUSION_SELF.fusion_get_aidlink());
            }
        }
        if (check_get('action') && (get('action') == "delete") && check_get('hday_id', FILTER_VALIDATE_INT)) {
            $cat_id = get('hday_id');

            if (dbcount("(fh_hday_id)", DB_FHCAL_HOLIDAYS, "fh_hday_id = :catid", [':catid' => $cat_id])) {
                dbquery("DELETE FROM ".DB_FHCAL_HOLIDAYS." WHERE fh_hday_id=:catid", [':catid' => $cat_id]);
                addnotice('success', self::$locale['FCAL_119']);
            }

            redirect(clean_request('', ['ref', 'action', 'hday_id'], FALSE));
        }
        $this->holiday_data += $this->calholiday;
        $this->contentForm();
    }

    private function holidaysQuery() {
        $result = dbquery($this->sqlCal->getHolidayQuery());
        if (dbrows( $result )) {
            while ($data = dbarray( $result )) {
                $this->holiday_data[] = $data;
            }
        } else {
            $this->holiday_data = [];
        }
        $this->holidayListing();
    }

    private function contentForm() {
        if ( check_post( 'savesettings' ) ) {
            $holiday_data = [
                'fh_hday_id'   => sanitizer( 'fh_hday_id', '', 'fh_hday_id' ),
                'fh_hday_name' => sanitizer( 'fh_hday_name', '', 'fh_hday_name' ),
                'fh_hday_date' => sanitizer( 'fh_hday_date', '', 'fh_hday_date' )
            ];

            if ( fusion_safe() ) {
                $dataid = dbquery_insert( DB_FHCAL_HOLIDAYS, $holiday_data, empty( $holiday_data['fh_hday_id'] ) ? 'save' : 'update' );
                addnotice( 'success', empty( $holiday_data['fh_hday_id'] ) ? self::$locale['FCAL_120'] : self::$locale['FCAL_121'] );
                redirect( clean_request( '', [ 'action', 'ref' ], FALSE ) );
            }
        }

        echo "<div class='mb-5'><h5>" . self::$locale['FCAL_127'] . "</h5></div>";
        echo openform( 'settingsFrm', 'POST' );
        echo form_hidden('fh_hday_id', '', $this->holiday_data['fh_hday_id']);
        echo form_text( 'fh_hday_name', self::$locale['FCAL_122'], $this->holiday_data['fh_hday_name'], [
            'inline'     => TRUE,
            'required'   => TRUE,
            'max_length' => 255
        ] );
        echo form_datepicker( 'fh_hday_date', self::$locale['FCAL_123'], $this->holiday_data['fh_hday_date'], [
            'inline'          => TRUE,
            'required'        => TRUE,
            'type'            => 'date',
            'placeholder'     => self::$locale['FCAL_124'],
            'showTime'        => FALSE,
            'date_format_js'  => 'YYYY-M-DD',
            'date_format_php' => 'Y-m-d',
            'width'           => '100%'
        ] );
        echo form_button( 'savesettings', self::$locale['submit'], self::$locale['submit'], [ 'class' => 'btn-primary' ] );
        echo closeform();
    }

    private function holidayListing() {
        echo openform('calendar_filter', 'post', FUSION_REQUEST);
        echo "<div class='m-b-5'>";
        echo "<div class='clearfix'>";
            echo "<div class='text-right'>";
                echo "<a class='btn btn-success btn-sm' href=" . clean_request( "ref=holidaysform", ["ref"], FALSE ) . "><i class='fa fa-fw fa-plus'></i>" . self::$locale['FCAL_125'] . "</a>";
            echo "</div>";
        echo closeform();
        if (!empty($this->holiday_data)) {
            echo "<div class='table-responsive'>";
                echo "<table class='table table-striped'>";
                    echo "<thead>";
                    echo "<tr>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_122'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_123'] . "</th>";
                        echo "<th class='text-center'></th>";
                    echo "</tr>";
                    echo "</thead>";
                    echo "<tbody>\n";
                    foreach ( $this->holiday_data as $cdata ) {
                            $edit_link = clean_request("section=holidays&ref=holidaysform&action=edit&hday_id=".$cdata['fh_hday_id'], ['section', 'ref', 'action', 'hday_id'], FALSE);
                            $delete_link = clean_request("section=holidays&ref=holidaysform&action=delete&hday_id=".$cdata['fh_hday_id'], ['section', 'ref', 'action', 'hday_id'], FALSE);

                        echo "<tr>";
                                echo "<td class='text-center'>" . $cdata['fh_hday_name'] . "</td>\n";
                                echo "<td class='text-center'>".date( 'Y-m-d', strtotime( $cdata['fh_hday_date'] ) )."</td>\n";
                            echo "<td class='text-center'><a href='".$edit_link."'><i class='fa fa-edit fa-fw'></i></a> <a href='".$delete_link."'><i class='fa fa-trash fa-fw'></i></a></td>\n";
                        echo "</tr>";
                    }
                    echo "</tbody>";
                echo "</table>";
            echo "</div>";

        } else {
            echo "<div class='well text-center'>" . self::$locale['FCAL_126'] . "</div>";
        }
        echo "</div>";
        echo "</div>";
    }


}
