<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://www.phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/admin_event.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

class CalEventAdmin extends CalServer {
    private static $instance = NULL;
    private $event_data = [];
    private $fhyear;
    private $fhmonth;
    private $calendar_data;
    private $caldata = [
        'fh_ev_id'         => '',
        'fh_ev_owner'      => 0,
        'fh_ev_visibility' => '',
        'fh_cat_id'        => 0,
        'fh_ev_subject'    => '',
        'fh_ev_desc'       => '',
        'fh_ev_readonly'   => 0,
        'fh_ev_ctime'      => '',
        'fh_ev_mtime'      => '',
        'fh_evall_day'     => 0,
        'fh_ev_activ'      => 0,
        'fh_o_id'          => '',
        'fh_ev_id'         => 0,
        'fh_o_start_date'  => '',
        'fh_o_end_date'    => '',
        'fh_o_start_ts'    => '',
        'fh_o_end_ts'      => '',
        'fh_o_time_type'   => 0
    ];

    public function __construct() {
        parent::__construct();
        $this->fhmonth = check_post( 'fh_month' ) ? post( 'fh_month' ) : date( 'm' );
        $this->fhyear = check_post( 'fh_year' ) ? post( 'fh_year' ) : date( 'Y' );
    }

    public static function getInstance() {
        if ( self::$instance == NULL ) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function startAdmin() {
        if ( check_get( 'cancel' ) ) {
            redirect( FUSION_SELF.fusion_get_aidlink() );
        }

        if ( check_get( 'ref' ) && get( 'ref' ) == "calendarform" ) {
            $this->calendarForm();
        } else {
            if ( isset( $this->fhmonth ) or ( isset( $this->fhyear ) ) ) {
                $this->calendarSelectedListing();
            }
            $this->calendarListing();
        }
    }

    private function calendarForm() {
        if ( check_get( 'action' ) && ( get( 'action' ) == "edit" ) && ( check_get( 'event_id', FILTER_VALIDATE_INT ) ) ) {
            $result = dbquery( $this->sqlCal->getEventQuery_1( (int)get( 'event_id' ) ) );
            if ( dbrows( $result ) ) {
                $this->event_data = dbarray( $result );
            } else {
                redirect( FUSION_SELF.fusion_get_aidlink() );
            }
        }
        $this->event_data += $this->caldata;
        $this->contentForm();
    }

    private function calendarSelectedListing() {
        if ( isset( $this->fhmonth ) && isset( $this->fhyear ) ) {
        	$startyear = check_post( 'fh_year', FILTER_VALIDATE_INT ) ? sanitizer( 'fh_year', '0', 'fh_year' ) : $this->fhyear;
        	$startmonth = check_post( 'fh_month', FILTER_VALIDATE_INT ) ? sanitizer( 'fh_month', '0', 'fh_month' ) : $this->fhmonth;
            $result = dbquery( $this->sqlCal->getEventQuery_1( '', $startyear, $startmonth ) );
            if ( dbrows( $result ) ) {
                while ( $data = dbarray( $result ) ) {
                    $this->calendar_data[] = $data;
                }
            } else {
            	$this->calendar_data = [];
            }
        }
    }

    private function contentForm() {

        if ( check_post( 'savesettings' ) ) {
            $event_data = [
                'fh_ev_id'         => sanitizer( 'fh_ev_id', '', 'fh_ev_id' ),
                'fh_ev_owner'      => !empty( $this->event_data['fh_ev_owner'] ) ? $this->event_data['fh_ev_owner'] : fusion_get_userdata( 'user_id' ),
                'fh_ev_visibility' => sanitizer( ['fh_ev_visibility'], '0', 'fh_ev_visibility' ),
                'fh_cat_id'        => sanitizer( 'fh_cat_id', '0', 'fh_cat_id' ),
                'fh_ev_subject'    => sanitizer( 'fh_ev_subject', '', 'fh_ev_subject' ),
                'fh_ev_desc'       => sanitizer( 'fh_ev_desc', '', 'fh_ev_desc' ),
                'fh_ev_readonly'   => check_post( 'fh_ev_readonly' ) ? 1 : 0,
                'fh_ev_ctime'      => empty( $this->event_data['fh_ev_ctime'] ) ? date( 'Y-m-d H:i:s', time() ) : $this->event_data['fh_ev_ctime'],
                'fh_ev_mtime'      => !empty( $this->event_data['fh_ev_id'] ) ? date( 'Y-m-d H:i:s', time() ) : '',
                'fh_evall_day'     => sanitizer( 'fh_evall_day', '0', 'fh_evall_day' ),
                'fh_ev_activ'      => check_post( 'fh_ev_activ' ) ? 1 : 0,
            ];
            $start_date = sanitizer( 'fh_o_start_date', '0', 'fh_o_start_date' );
            $end_date = sanitizer( 'fh_o_end_date', '0', 'fh_o_end_date' );
            $fh_o_start_date = !empty( $event_data['fh_evall_day'] ) ? date( 'Y-m-d', $start_date ) : '';
            $fh_o_end_date = !empty( $event_data['fh_evall_day']) ? date( 'Y-m-d', $end_date ) : '';
            $fh_o_start_ts = empty( $event_data['fh_evall_day']) ? date( 'Y-m-d H:i:s', $start_date ) : '';
            $fh_o_end_ts = empty( $event_data['fh_evall_day']) ? date( 'Y-m-d H:i:s', $end_date ) : '';
            $occurrences = [
                'fh_o_id'         => sanitizer( 'fh_o_id', '', 'fh_o_id' ),
                'fh_ev_id'        => sanitizer( 'fh_ev_id', '', 'fh_ev_id' ),
                'fh_o_start_date' => $fh_o_start_date,
                'fh_o_end_date'   => $fh_o_end_date,
                'fh_o_start_ts'   => $fh_o_start_ts,
                'fh_o_end_ts'     => $fh_o_end_ts,
                'fh_o_time_type'  => $event_data['fh_evall_day']
            ];

            if ( fusion_safe() ) {
                $dataid = dbquery_insert( DB_FHCAL_EVENTS, $event_data, empty( $event_data['fh_ev_id'] ) ? 'save' : 'update' );
                $occurrences['fh_ev_id'] = empty( $occurrences['fh_ev_id'] ) ? $dataid : $occurrences['fh_ev_id'];
                dbquery_insert( DB_FHCAL_OCCURACE, $occurrences, empty( $occurrences['fh_o_id'] ) ? 'save' : 'update' );
                addnotice( 'success', empty( $event_data['fh_ev_id'] ) ? self::$locale['FCAL_134'] : self::$locale['FCAL_135'] );
                redirect( clean_request( '', [ 'event_id', 'action', 'ref' ], FALSE ) );
            }
        }

        $fh_o_start_date = !empty($this->event_data['fh_evall_day']) ? $this->event_data['fh_o_start_date'] : $this->event_data['fh_o_start_ts'];
        $fh_o_end_date = !empty($this->event_data['fh_evall_day']) ? $this->event_data['fh_o_end_date'] : $this->event_data['fh_o_end_ts'];

        $result = dbquery($this->sqlCal->getCategoryQuery());
        if (dbrows($result)) {
            while ($data = dbarray($result)) {
                $catlist[$data['fh_cat_id']] = $data['fh_cat_name'];
            }
        }
        $admin_list = fusion_get_groups();
        echo openform( 'settingsFrm', 'POST' );
        echo form_hidden('fh_ev_id', '', $this->event_data['fh_ev_id']);
        echo form_hidden('fh_o_id', '', $this->event_data['fh_o_id']);
        echo form_hidden('fh_ev_owner', '', $this->event_data['fh_ev_owner']);
        echo form_text( 'fh_ev_subject', self::$locale['FCAL_031'], $this->event_data['fh_ev_subject'], [
            'inline'     => TRUE,
            'required'   => TRUE,
            'max_length' => $this->eventsettings['fh_subject_max']
        ] );
        echo form_textarea( 'fh_ev_desc', self::$locale['FCAL_032'], $this->event_data['fh_ev_desc'], [
            'inline'   => TRUE,
            'autosize' => TRUE
        ] );
        echo form_datepicker( 'fh_o_start_date', self::$locale['FCAL_033'], $fh_o_start_date, [
            'inline'      => TRUE,
            'required'    => TRUE,
            'placeholder' => self::$locale['FCAL_034'],
            'join_to_id'  => 'fh_o_end_date',
            'width'       => '100%'
        ] );
        echo form_datepicker( 'fh_o_end_date', self::$locale['FCAL_035'], $fh_o_end_date, [
            'inline'       => TRUE,
            'required'     => TRUE,
            'placeholder'  => self::$locale['FCAL_036'],
            'join_from_id' => 'fh_o_start_date',
            'width'        => '100%'
        ] );
        echo form_select('fh_cat_id', self::$locale['FCAL_037'], $this->event_data['fh_cat_id'], [
            'inline'     => TRUE,
            'required'   => TRUE,
            'options'    => $catlist
        ]);
        echo form_select( 'fh_evall_day', self::$locale['FCAL_038'], $this->event_data['fh_evall_day'], [
            'options'    => ['0' => self::$locale['FCAL_039'], '1' => self::$locale['FCAL_040'], '2' => self::$locale['FCAL_041']],
            'inline'     => TRUE
        ] );
        echo form_select( 'fh_ev_visibility[]', self::$locale['FCAL_042'], $this->event_data['fh_ev_visibility'], [
            'inline'      => TRUE,
            'options'     => $admin_list,
            'placeholder' => self::$locale['choose'],
            'multiple'    => TRUE
        ] );
        echo form_checkbox( 'fh_ev_readonly', self::$locale['FCAL_043'], $this->event_data['fh_ev_readonly'], [
            'inline' => TRUE,
            'toggle' => TRUE
        ] );
        echo form_checkbox( 'fh_ev_activ', self::$locale['FCAL_136'], $this->event_data['fh_ev_activ'], [
            'inline' => TRUE,
            'toggle' => TRUE
        ] );
        echo form_button( 'savesettings', self::$locale['submit'], self::$locale['submit'], [ 'class' => 'btn-primary' ] );
        echo closeform();
    }

    private function calendarListing() {
        $calendar_cats = dbcount("(fh_cat_id)", DB_FHCAL_CATEGORY);
        echo "<div class='m-b-5'>";
        echo "<div class='clearfix'>";
            echo "<div class='text-right'>";
                if ($calendar_cats) {
                    echo "<a class='btn btn-success btn-sm' href=" . clean_request( "ref=calendarform", ["ref"], FALSE ) . "><i class='fa fa-fw fa-plus'></i>" . self::$locale['FCAL_137'] . "</a>";
                }
            echo "</div>";
        echo openform('eventadminform', 'post', FUSION_REQUEST, ['max_tokens' => 1]);
        echo "<div class='row'>";
            echo "<div class='col-xs-12 col-sm-6'>";
                echo form_select( 'fh_year', '', $this->fhyear, [
                    'options'     => range(date('Y')-3, date('Y')+3),
                    'keyflip'     => TRUE,
                    'allowclear'  => TRUE,
                    'inner_width' => '100px',
                    'width'       => '100px',
                    'class'       => 'pull-right',
                    'onchange'    => 'document.eventadminform.submit()'
                ] );
            echo "</div>";
            echo "<div class='col-xs-12 col-sm-6'>";
                echo form_select( 'fh_month', '', $this->fhmonth, [
                    'options'     => self::setFullMonths(),
                    'inner_width' => '140px',
                    'width'       => '140px',
                    'class'       => 'pull-left',
                    'onchange'    => 'document.eventadminform.submit()'
                ] );
            echo "</div>";
        echo "</div>";
        echo closeform();
        if ( !empty( $this->calendar_data ) ) {
            echo "<div class='table-responsive'>";
                echo "<table class='table table-striped'>";
                    echo "<thead>";
                    echo "<tr>";
                        echo "<th class='text-center'></th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_031'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_060'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_032'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_042'] . "</th>";
                        echo "<th class='text-center'>" . self::$locale['FCAL_138'] . "</th>";
                        echo "<th class='text-center'></th>";
                    echo "</tr>";
                    echo "</thead>";
                    echo "<tbody>\n";
                    foreach ( $this->calendar_data as $cdata ) {
                        $edit_link = clean_request( "section=calendar&ref=calendarform&action=edit&event_id=".$cdata['fh_ev_id'], ['section', 'ref', 'action', 'event_id'], FALSE );
                        $delete_link = clean_request( "section=calendar&ref=calendarform&action=delete&event_id=".$cdata['fh_ev_id'], ['section', 'ref', 'action', 'event_id'], FALSE );
                        $infstart['start'] = !empty( $cdata['fh_evall_day'] ) ? $cdata['fh_o_start_date'] : $cdata['fh_o_start_ts'];
                        $infend['start'] = !empty( $cdata['fh_evall_day'] ) ? $cdata['fh_o_end_date'] : $cdata['fh_o_end_ts'];
                        $hours24 = $this->eventsettings['fh_hours_24'] ? ' H:i:s' : ' h:i:s A';
                        $sdattim = !empty( $cdata['fh_evall_day'] ) ? '' : $hours24;
                        $edattim = !empty( $cdata['fh_evall_day'] ) ? '' : $hours24;

                        echo "<tr>";
                            echo "<td class='text-left'><i class='fa fa-square fa-2x' style='color:" . $cdata['fh_cat_color'] . ";' title=" .$cdata['fh_cat_name']. "></i></td>\n";
                            echo "<td class='text-left'>".$cdata['fh_ev_subject']."</td>\n";
                            echo "<td class='text-left'>
                                <div class='row'>
                                    <div class='col-xs-12 col-sm-5 text-right'>".date( $this->setFormatDate().$sdattim, self::get_timestamp( $infstart ) )."</div>
                                    <div class='col-xs-12 col-sm-2 text-center'> - </div>
                                    <div class='col-xs-12 col-sm-5 text-left'>".date( $this->setFormatDate().$edattim, self::get_timestamp( $infend ) )."</div>
                                </div>
                            </td>\n";
                            echo "<td class='text-left'>".$cdata['fh_ev_desc']."</td>\n";
                            echo "<td class='text-left'>".getgroupname( $cdata['fh_ev_visibility'] )."</td>\n";
                            echo "<td class='text-left'><span class='badge'>" . ( $cdata['fh_ev_activ'] ? self::$locale['FCAL_139'] : self::$locale['FCAL_140'] ) . "</span></td>\n";
                            echo "<td class='text-center'><a href='".$edit_link."'><i class='fa fa-edit fa-fw'></i></a> <a href='".$delete_link."'><i class='fa fa-trash fa-fw'></i></a></td>\n";
                        echo "</tr>";
                    }
                    echo "</tbody>";
                echo "</table>";
            echo "</div>";
        } else {
            echo "<div class='well text-center'>" . self::$locale['FCAL_141'] . "</div>";
        }
        echo "</div>";
        echo "</div>";
    }

}
