<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://www.phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/admin_settings.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

class CalSettings extends CalServer {
    private static $instance = NULL;

    public function __construct() {
        parent::__construct();
    }

    public static function getInstance() {
        if (self::$instance == NULL) {
            self::$instance = new static();
        }

        return self::$instance;
    }

    public function startAdmin() {
        if (check_post('savesettings')) {
            $inputArray = [
                'fh_week_start'        => sanitizer('fh_week_start', '0', 'fh_week_start'),
                'fh_hours_24'          => check_post('fh_hours_24') ? 1 : 0,
                'fh_date_format'       => sanitizer('fh_date_format', '0', 'fh_date_format'),
                'fh_subject_max'       => sanitizer('fh_subject_max', '50', 'fh_subject_max'),
                'fh_birthdate'         => sanitizer('fh_birthdate', '0', 'fh_birthdate'),
                'fh_holidays'          => sanitizer('fh_holidays', '0', 'fh_holidays'),
                'fh_submission'        => sanitizer('fh_submission', '0', 'fh_submission'),
                'fh_submission_access' => sanitizer(['fh_submission_access'], USER_LEVEL_MEMBER, 'fh_submission_access')
            ];
            if (fusion_safe()) {
                foreach ($inputArray as $settings_name => $settings_value) {
                    $inputSettings = [
                        'settings_name'  => $settings_name,
                        'settings_value' => $settings_value,
                        'settings_inf'   => "fh_calendar_panel"
                    ];
                    dbquery_insert(DB_SETTINGS_INF, $inputSettings, "update", ["primary_key" => "settings_name"]);
                }
                addnotice("success", self::$locale['FCAL_104']);
                redirect(clean_request("section=settings", ["", "aid"]));
            }
        }
        opentable(self::$locale['FCAL_103']);
        echo openform('settingsform', 'post', FUSION_REQUEST);
        echo form_select('fh_week_start', self::$locale['FCAL_105'], $this->eventsettings['fh_week_start'], [
            'options'    => [0 => self::$locale['FCAL_106'], 1 => self::$locale['FCAL_107'], 6 => self::$locale['FCAL_108']],
            'inline'     => TRUE
        ]);
        echo form_checkbox('fh_hours_24', self::$locale['FCAL_109'], $this->eventsettings['fh_hours_24'], [
            'inline' => TRUE,
            'toggle' => TRUE
        ]);
        echo form_text('fh_subject_max', self::$locale['FCAL_110'], $this->eventsettings['fh_subject_max'], [
            'inline'      => TRUE,
            'max_length'  => 4,
            'type'        => 'number'
        ]);
        echo form_select('fh_date_format', self::$locale['FCAL_111'], $this->eventsettings['fh_date_format'], [
            'options'    => [0 => self::$locale['FCAL_112'], 1 => self::$locale['FCAL_113'], 2 => self::$locale['FCAL_114']],
            'inline'     => TRUE
        ]);
        echo form_checkbox('fh_submission', self::$locale['FCAL_115'], $this->eventsettings['fh_submission'], [
            'inline' => TRUE,
            'toggle' => TRUE
        ]);
        echo form_select('fh_submission_access[]', self::$locale['submit_access'], $this->eventsettings['fh_submission_access'], [
            'inline'   => TRUE,
            'options'  => fusion_get_groups([USER_LEVEL_PUBLIC]),
            'multiple' => TRUE,
        ]);
        tablebreak();
        $deact = TRUE;
        if (column_exists(DB_USERS, 'user_birthdate')) {
            $deact = FALSE;
        }
        echo form_checkbox('fh_birthdate', self::$locale['FCAL_116'], !empty( $deact ) ? 0 : $this->eventsettings['fh_birthdate'], [
            'inline'     => TRUE,
            'ext_tip'    => self::$locale['FCAL_117'],
            'deactivate' => $deact,
            'toggle'     => TRUE
        ]);
        echo form_checkbox('fh_holidays', self::$locale['FCAL_118'], $this->eventsettings['fh_holidays'], [
            'inline' => TRUE,
            'toggle' => TRUE
        ]);
        echo form_button('savesettings', self::$locale['save'], self::$locale['save'], ['class' => 'btn-primary']);
        echo closeform();
        closetable();
    }
}
