<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/events.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

class CalEvent extends CalServer {
    private static $instance = NULL;

    public function __construct() {
        parent::__construct();
        $this->mcode = check_get( 'mcode' ) ? get( 'mcode' ) : '';
        $this->eventid = check_get( 'event_id' ) ? get( 'event_id' ) : '';
        add_to_title( self::$locale['FCAL_003'] );
    }

    public static function getInstance() {
        if ( self::$instance === NULL ) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function DisplayEvents() {
    	if (check_get('action') && get('action') == 'now') {
    		self::eventsForm();
    	} else if (check_post('savecalendar')) {
            $event_data = [
                'fh_ev_id'         => sanitizer('fh_ev_id', '', 'fh_ev_id'),
                'fh_ev_owner'      => !empty($this->event_data['fh_ev_owner']) ? $this->event_data['fh_ev_owner'] : fusion_get_userdata('user_id'),
                'fh_ev_visibility' => sanitizer(['fh_ev_visibility'], '0', 'fh_ev_visibility'),
                'fh_cat_id'        => sanitizer('fh_cat_id', '0', 'fh_cat_id'),
                'fh_ev_subject'    => sanitizer('fh_ev_subject', '', 'fh_ev_subject'),
                'fh_ev_desc'       => sanitizer('fh_ev_desc', '', 'fh_ev_desc'),
                'fh_ev_readonly'   => check_post('fh_ev_readonly') ? 1 : 0,
                'fh_ev_ctime'      => empty($this->event_data['fh_ev_ctime']) ? date('Y-m-d H:i:s', time()) : $this->event_data['fh_ev_ctime'],
                'fh_ev_mtime'      => !empty($this->event_data['fh_ev_id']) ? date('Y-m-d H:i:s', time()) : '',
                'fh_evall_day'     => sanitizer('fh_evall_day', '0', 'fh_evall_day'),
                'fh_ev_activ'      => iADMIN ? 1 : 0,
            ];
            $start_date = sanitizer('fh_o_start_date', '0', 'fh_o_start_date');
            $end_date = sanitizer('fh_o_end_date', '0', 'fh_o_end_date');
            $fh_o_start_date = !empty($event_data['fh_evall_day']) ? date('Y-m-d', $start_date) : '';
            $fh_o_end_date = !empty($event_data['fh_evall_day']) ? date('Y-m-d', $end_date) : '';
            $fh_o_start_ts = empty($event_data['fh_evall_day']) ? date('Y-m-d H:i:s', $start_date) : '';
            $fh_o_end_ts = empty($event_data['fh_evall_day']) ? date('Y-m-d H:i:s', $end_date) : '';
            $occurrences = [
                'fh_o_id'         => sanitizer('fh_o_id', '', 'fh_o_id'),
                'fh_ev_id'        => sanitizer('fh_ev_id', '', 'fh_ev_id'),
                'fh_o_start_date' => $fh_o_start_date,
                'fh_o_end_date'   => $fh_o_end_date,
                'fh_o_start_ts'   => $fh_o_start_ts,
                'fh_o_end_ts'     => $fh_o_end_ts,
                'fh_o_time_type'  => $event_data['fh_evall_day']
            ];

            if ( fusion_safe() ) {
                $dataid = dbquery_insert( DB_FHCAL_EVENTS, $event_data, 'save' );
                $occurrences['fh_ev_id'] = empty($occurrences['fh_ev_id']) ? $dataid : $occurrences['fh_ev_id'];
                dbquery_insert( DB_FHCAL_OCCURACE, $occurrences, 'save' );
                addnotice( 'success', self::$locale['FCAL_050'] );
                redirect( clean_request( '', [ 'event_id', 'action', 'year', 'month', 'event_day' ], FALSE ) );
            }
    	} else {
    	    $ev_data = $this->eventsQuery();
    	    $context = [
    	        'stablename' => self::$locale['FCAL_051'],
    	        'month'      => $ev_data['months'],
    	        'nodata'     => $ev_data['textday'],
    	        'texthead'   => $ev_data['texthead'],
    	        'submitlink' => (iMEMBER && $this->eventsettings['fh_submission'] && checkgroup( $this->eventsettings['fh_submission_access'] ) && empty( get( 'mcode' ) ) ) ? "<a class='btn btn-success btn-sm' href='" . FCAL_PATH . "fh_events.php?action=now&year=". get('year') . "&month=" . get('month') . "&event_day=" . get('event_day') . "'><i class='fa fa-fw fa-plus'></i>" . self::$locale['FCAL_052'] . "</a>" : ''
    	    ];
    	    $info = [];
    	    if ( dbrows( $ev_data['query'] ) ) {
    	        $context['nodata'] = '';
    	        while ( $data = dbarray( $ev_data['query'] ) ) {
    	            $infstart['start'] = !empty($data['fh_evall_day']) ? $data['fh_o_start_date'] : $data['fh_o_start_ts'];
    	            $infend['start'] = !empty($data['fh_evall_day']) ? $data['fh_o_end_date'] : $data['fh_o_end_ts'];

    	            $hours24 = $this->eventsettings['fh_hours_24'] ? ' H:i:s' : ' h:i:s A';
    	            $sdattim = !empty($data['fh_evall_day']) ? '' : $hours24;
    	            $edattim = !empty($data['fh_evall_day']) ? '' : $hours24;

    	            $data['fh_allday'] = !empty($data['fh_evall_day']) ? self::$locale['FCAL_053'] : self::$locale['FCAL_054'];
    	            $data['fh_start'] = date($this->setFormatDate().$sdattim, self::get_timestamp($infstart));
    	            $data['fh_stop'] = date($this->setFormatDate().$edattim, self::get_timestamp($infend));
    	            //$data['edit_icon'] = iADMIN ? "<i class='fas fa-edit'></i>" : '';
    	            //$data['del_icon'] = iADMIN ? "<i class='fas fa-trash'></i>" : '';
    	            $data['editdata'] = "href='". FCAL_PATH ."fh_events.php?event_id=". $data['fh_ev_id'] ."' title='".$data['fh_ev_desc']."'>";
    	            $info[] = $data;
    	        }
    	    }
    	    $context['data'] = $info;
    	    scheduled($context);
    	    //echo fusion_render(FCAL_PATH.'twig/', 'scheduled.twig', $context, TRUE);
        }
    }

    private function eventsForm() {
        $result = dbquery($this->sqlCal->getCategoryQuery());
        if (dbrows($result)) {
            while ($data = dbarray($result)) {
                $catlist[$data['fh_cat_id']] = $data['fh_cat_name'];
            }
        }
        $admin_list = fusion_get_groups();
        opentable(self::$locale['FCAL_030']);
        echo openform( 'settingsFrm', 'POST', FCAL_PATH ."fh_events.php");
        echo form_hidden('fh_ev_id', '', '');
        echo form_hidden('fh_o_id', '', '');
        echo form_hidden('fh_ev_owner', '', fusion_get_userdata('user_id'));
        echo form_text( 'fh_ev_subject', self::$locale['FCAL_031'], '', [
            'inline'     => TRUE,
            'required'   => TRUE,
            'max_length' => $this->eventsettings['fh_subject_max']
        ] );
        echo form_textarea( 'fh_ev_desc', self::$locale['FCAL_032'], '', [
            'inline'   => TRUE,
            'autosize' => TRUE
        ] );
        echo form_datepicker( 'fh_o_start_date', self::$locale['FCAL_033'], get('year') . '-' . get('month') . '-' . get('event_day') , [
            'inline'      => TRUE,
            'required'    => TRUE,
            'placeholder' => self::$locale['FCAL_034'],
            'join_to_id'  => 'fh_o_end_date',
            'width'       => '100%'
        ] );
        echo form_datepicker( 'fh_o_end_date', self::$locale['FCAL_035'], '', [
            'inline'       => TRUE,
            'required'     => TRUE,
            'placeholder'  => self::$locale['FCAL_036'],
            'join_from_id' => 'fh_o_start_date',
            'width'        => '100%'
        ] );
        echo form_select('fh_cat_id', self::$locale['FCAL_037'], 0, [
            'inline'     => TRUE,
            'required'   => TRUE,
            'options'    => $catlist
        ]);
        echo form_select( 'fh_evall_day', self::$locale['FCAL_038'], 0, [
            'options'    => ['0' => self::$locale['FCAL_039'], '1' => self::$locale['FCAL_040'], '2' => self::$locale['FCAL_041']],
            'inline'     => TRUE
        ] );
        echo form_select( 'fh_ev_visibility[]', self::$locale['FCAL_042'], 0, [
            'inline'      => TRUE,
            'options'     => $admin_list,
            'placeholder' => self::$locale['choose'],
            'multiple'    => TRUE
        ] );
        echo form_checkbox( 'fh_ev_readonly', self::$locale['FCAL_043'], 0, [
            'inline' => TRUE,
            'toggle' => TRUE
        ] );
        echo form_button( 'savecalendar', self::$locale['submit'], self::$locale['submit'], [ 'class' => 'btn-primary' ] );
        echo closeform();
        closetable();
    }

    private function eventsQuery() {
        $eventid = '';
        $year = date( 'Y' );
        $month = '';
        $day = '';
    	if ( !empty( $this->eventid ) ) {
            $fhmonths = '';
            $eventid = $this->eventid;
            $textday = self::$locale['FCAL_055'];
            $texthead = self::$locale['FCAL_020'];
    	} else if ( !empty( $this->mcode ) ) {
            $fhmonths = self::setFullMonths( $this->mcode );
            $textday = sprintf( self::$locale['FCAL_056'], $fhmonths );
            $texthead = sprintf( self::$locale['FCAL_057'], $fhmonths.self::$locale['FCAL_059'] );
            $eventid = '';
            $month = $this->mcode;
    	} else if (!empty($this->fmonth) && !empty($this->fday)) {
            $fhmonths = self::setFullMonths( $this->fmonth );
            $textday = self::$locale['FCAL_064'];
            $texthead = sprintf( self::$locale['FCAL_057'], self::$locale['FCAL_058'] );
            $eventid = '';
            $month = $this->fmonth;
            $day = $this->fday;
    	} else {
            $fhmonths = self::setFullMonths( $this->fmonth );
            $textday = sprintf( self::$locale['FCAL_056'], $fhmonths );
            $texthead = sprintf( self::$locale['FCAL_057'], $fhmonths.self::$locale['FCAL_059'] );
            $eventid = '';
            $year = $this->fyear;
            $month = $this->fmonth;
            $day = 1;
    	}
        $info = [
            'months'   => $fhmonths,
            'textday'  => $textday,
            'texthead' => $texthead,
            'query'    => dbquery( $this->sqlCal->getEventQuery_1( $eventid, $year, $month, $day ) )
        ];
    	return $info;
    }
}