<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/server.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\FhCal;

abstract class CalServer {
    protected static $locale = [];
    public $sqlCal = NULL;
    public $fmonth = NULL;
    public $fyear = NULL;
    public $fday = NULL;
    public $dayName = NULL;
    public $category = NULL;
    public $calinf = [];
    public $eventsettings;

    public function __construct() {
        require_once FCAL_PATH."templates/calendar.tpl.php";
        add_to_head("<link rel='stylesheet' href='" . FCAL_PATH . "css/calendar.css' type='text/css' media='all' />");
        $this->sqlCal = new CalSql();
        self::$locale = fusion_get_locale( "", FCAL_LOCALE );
        self::callSetup();
        self::getCategory();
        $this->fmonth = check_get( 'month' ) ? get( 'month' ) : date( 'n' );
        $this->fyear = check_get( 'year' ) ? get( 'year' ) : date( 'Y' );
        $this->fday = check_get( 'event_day' ) ? get( 'event_day' ) : date( 'd' );
    }
    // setup kell
    public function callSetup() {
        $this->eventsettings = get_settings("fh_calendar_panel");
    }
    //het kezdo napja
    public function weekStartDay() {
        return $this->eventsettings['fh_week_start'];
    }

    public function getCategory() {
        $result = dbquery( $this->sqlCal->getCategoryQuery( ) );
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $this->category[$data['fh_cat_id']] = $data['fh_cat_color'];
            }
        }
    }

    //elso nap
    public function setFirstDay($month, $year ) {
        $first_day = 1 - self::dayOfWeek($month, 1, $year);
        return  mktime(0, 0, 0, $month, $first_day, $year);
    }
    //utolso nap
    public function setLastDay($month, $year ) {
        $last_day = self::setEmptyDay($month, $year ) * 7 - self::dayOfWeek($month, 1, $year);
        return mktime(23, 59, 59, $month, $last_day+1, $year);
    }
    //het napja
	public function dayOfWeek($month, $day, $year) {
	    return self::dayOfWeekTs(mktime(0, 0, 0, $month, $day, $year));
	}

	public function dayOfWeekTs($timestamp) {
	    $days = date('w', $timestamp);
	    return ($days + 7 - self::weekStartDay()) % 7;
	}

	public function allDay($startst, $stopst) {
		$diff = $stopst - $startst;
		if ( $diff > 0 ) {
		    $add_days = floor( $diff / 86400 );
		} else {
		    $add_days = 0;
		}
		return $add_days;
	}

    public function fhDateWeek($timestamp = 0) {
        return date( "W", $timestamp );
    }

    //honap neve
    public function setFullMonths( $key = NULL ) {
        $fmonth = explode( "|", self::$locale['months'] );
        return isset( $key ) ? $fmonth[$key] : $fmonth;
    }

    public function setEmptyDay ($month, $year ) {
        return self::weeksInMonth($month, $year);
    }

	public function weeksInMonth($month, $year) {
	    $days = self::daysInMonth($month, $year);
	    $days_before_month = self::dayOfWeek($month, 1, $year);
	    $days_after_month = 6 - self::dayOfWeek($month, $days, $year);
	    return ($days_before_month + $days + $days_after_month) / 7;
	}

	public function daysInMonth($month, $year) {
	    return date( 't', mktime(0, 0, 0, $month, 1, $year) );
	}

	public function setDaysInMonth($month, $day, $year) {
	    return date( 'd', mktime(0, 0, 0, $month, $day, $year) );
	}

    //honap ev szammal
    public function calculateMonth( $key = NULL ) {
        $info = [
            'prev_year'  => $this->fmonth == 1 ? $this->fyear - 1 : $this->fyear,
            'next_year'  => $this->fmonth == 12 ? $this->fyear + 1 : $this->fyear,
            'prev_month' => $this->fmonth == 1 ? 12 : $this->fmonth - 1,
            'next_month' => $this->fmonth == 12 ? 1 : $this->fmonth + 1
        ];
        return !empty( $key ) ? $info[$key] : $info;
    }

    //hetvege + het napjai
    public function setDays() {
        $inf = [0 => 'W'];
        for($i = 0; $i < 7; $i++) {
            $d = ( $i + $this->eventsettings['fh_week_start'] ) % 7;
            $inf[] = self::day_name( $d );
        }
        return $inf;
    }

    //napok neve
    public function day_name( $day ) {
        $day = $day % 7;
        return $this->setDayName($day);
    }

    //het napjai roviden
    public function setDayName( $key = NULL ) {
        $fdays = explode( "|", self::$locale['FCAL_weekdays'] );
        return (isset( $key ) ? $fdays[$key] : $fdays);
    }

    public function get_timestamp($prefix, $hour = 0, $minute = 0, $second = 0) {
        if ( !empty( $prefix['start'] ) ) {
            if( preg_match( '/(\d+)[:\.](\d+)\s?(\w+)?/', $prefix['start'], $time_matches ) ) {
                $hour = $time_matches[1];
                $minute = $time_matches[2];
                if ( isset( $time_matches[3] ) ) {
                    $period = $time_matches[3];
                    if ( $hour == 12 ) {
                        $hour = 0;
                    }
                    if ( strcasecmp( "am", $period ) == 0 ) {
                        // AM
                    } else if(strcasecmp("pm", $period) == 0) {
                        $hour += 12;
                    } else {
                    }
                }
            }
        }

        if(preg_match('/(\d+)[\.\/\-\ ](\d+)[\.\/\-\ ](\d+)/', $prefix['start'], $date_matches)) {
            $month = $date_matches[2];
            $day = $date_matches[3];
            $year = $date_matches[1];
        }
        return mktime($hour, $minute, $second, $month, $day, $year);
    }

    public function setFormatDate() {
    	switch($this->eventsettings['fh_date_format']) {
		case 0: // Month Day Year
			$inf = 'm.d.Y';
			break;
		case 1: // Year Month Day
			$inf = 'Y.m.d';
			break;
		case 2: // Day Month Year
			$inf = 'd.m.Y';
			break;
		default:
			$inf = 'Y.m.d';
	}
	    return $inf;
    }

    public function setDayData($timestamp) {
        return date( 'Y-n-j', $timestamp );
    }

    public function getDateDay($key = '', $mont = 0, $day = 0, $year = 0) {
        $thismonth = mktime( 12, 0, 0, $mont, $day, $year );
        $gdate = getdate($thismonth);
        return isset( $key ) ? $gdate[$key] : $gdate;
    }

    public function getWork_Day( $year = FALSE, $month = FALSE ) {
        if ($this->eventsettings['fh_holidays']) {
            self::getHoli_Day(self::setDayData(self::setFirstDay( $month, $year )), self::setDayData(self::setLastDay( $month, $year )));
        }
        if (iMEMBER && $this->eventsettings['fh_birthdate']) {
            if (column_exists(DB_USERS, 'user_birthdate')) {
                self::getBirth_Date(self::setDayData(self::setFirstDay( $month, $year )), self::setDayData(self::setLastDay( $month, $year )));
            }
        }
        self::get_Event(self::setDayData(self::setFirstDay( $month, $year )), self::setDayData(self::setLastDay( $month, $year )));
    }

    public function getMotherDay($year = FALSE, $month = FALSE) {
        if ($month >= '5' && $month <= '6') {
            $year = empty( $year ) ? date( 'Y' ) : $year;
            $month = empty( $month ) ? date( 'm' ) : $month;
            $fd = 0;
            for ( $d = 0; $d <= 30; $d++ ) {
                $fday = mktime( 0, 0, 0, 5, 1, $year );
                $weekdate = date( 'Y-m-d', $fday +( $d * 86400 ) );
                $dateinfo = explode( "-", $weekdate );
                $wdayClass = self::getDateDay( 'wday', $dateinfo[1], $dateinfo[2], $dateinfo[0] );
                if ( $wdayClass == 0 ) {
                    $info[$dateinfo[2]] = $dateinfo[2];
                }
            }
            if ( !empty( $info ) ) {
                foreach ( $info as $fday ) {
                    if ( $fd == 0 ) {
                        $this->calinf['holidays']['05-' . $fday] = self::$locale['FCAL_011'];
                    }
                    $fd++;
                }
            }
        }
    }

    public function getFatherDay($year = FALSE, $month = FALSE) {
        if ($month >= '6' && $month <= '7') {
            $year = empty( $year ) ? date( 'Y' ) : $year;
            $month = empty( $month ) ? date( 'm' ) : $month;
            $fd = 0;
            for ( $d = 0; $d <= 30; $d++ ) {
                $fday = mktime( 0, 0, 0, 6, 1, $year );
                $weekdate = date( 'Y-m-d', $fday +( $d * 86400 ) );
                $dateinfo = explode( "-", $weekdate );
                $wdayClass = self::getDateDay( 'wday', $dateinfo[1], $dateinfo[2], $dateinfo[0] );
                if ( $wdayClass == 0 ) {
                    $info[$dateinfo[2]] = $dateinfo[2];
                }
            }
            if ( !empty( $info ) ) {
                foreach ( $info as $fday ) {
                    if ( $fd == 2 ) {
                        $this->calinf['holidays']['06-' . $fday] = self::$locale['FCAL_012'];
                    }
                    $fd++;
                }
            }
        }
    }

    /**
    * PHP function that always returns Easter Sunday
    *
    * @param bool|int $year - If it is FALSE, it will display the current year, if it is a number, then it will return the current year
    * @return string Easter Sunday in YYYY-MM-DD format.
    */
    function getEasterDay($year = FALSE, $month = FALSE) {
        if ($month >= '3' && $month <= '6') {
            $year = empty( $year ) ? date( 'Y' ) : $year;
            $month = empty( $month ) ? date( 'm' ) : $month;
            $easterday = easter_days( $year );
            $easter = mktime( 0, 0, 0, 3, 21 + $easterday, $year );

            $text = self::$locale['FCAL_065'] . "\n - ";
            $this->calinf['holidays'][date( 'm-d', strtotime( '-2 day', $easter ) )] = $text.self::$locale['FCAL_013'];
            $this->calinf['holidays'][date( 'm-d', $easter )] = $text.self::$locale['FCAL_014'];
            $this->calinf['holidays'][date( 'm-d', strtotime( '+1 day', $easter ) )] = $text.self::$locale['FCAL_015'];
            $this->calinf['holidays'][date( 'm-d', strtotime( '+49 days', $easter ) )] = $text.self::$locale['FCAL_016'];
            $this->calinf['holidays'][date( 'm-d', strtotime( '+50 days', $easter ) )] = $text.self::$locale['FCAL_017'];
        }
    }

    public function getHoli_Day($startdate, $enddate) {
        $startinfo = explode( "-", $startdate );
        $endinfo = explode( "-", $enddate );

        $stdat = "1900-".$startinfo[1]."-".$startinfo[2];
        $enddat = "2999-".$endinfo[1]."-".$endinfo[2];
        $info = [];
        $result = dbquery( $this->sqlCal->getHolidayQuery( ['condition' => "fh_hday_date BETWEEN CAST( '" . $stdat . "' AS DATE ) AND CAST( '" . $enddat . "' AS DATE )", 'order' => "fh_hday_id ASC"] ) );
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                if (!empty($data['fh_hday_date']) || $data['fh_hday_date'] != '1900-01-01' || $data['fh_hday_date'] != '1970-1-01') {
                    $text = self::$locale['FCAL_018'] . "\n - ";
                    $dateinfo = explode( "-", $data['fh_hday_date'] );
                    $text .= $data['fh_hday_name'] . ', ';
                    $info[$dateinfo[1] . '-' . $dateinfo[2]] = $text;
                }
            }
            $this->calinf['holidays'] = $info;
        }
        self::getEasterDay($this->fyear, $this->fmonth);
        $dateinfo = explode( "-", $startdate );
        if ($dateinfo[1] >= '4' && $dateinfo[1] <= '6') {
            self::getMotherDay( $this->fyear, $this->fmonth );
        }
        if ($dateinfo[1] >= '5' && $dateinfo[1] <= '7') {
            self::getFatherDay( $this->fyear, $this->fmonth );
        }
    }

    public function getBirth_Date( $startdate, $enddate ) {
        $startinfo = explode( "-", $startdate );
        $endinfo = explode( "-", $enddate );

        $info = [];
        $result = dbquery( $this->sqlCal->getBirthdayQuery( ['condition' => "MONTH( user_birthdate ) <= '" . $endinfo[1] . "' AND MONTH( user_birthdate ) >= '" . $startinfo[1] . "'", 'order' => "MONTH(user_birthdate), DAY(user_birthdate) ASC"] ) );
        if ( dbrows( $result ) ) {
            $text = self::$locale['FCAL_019'] . "\n - ";
            while ( $data = dbarray( $result ) ) {
                if (!empty( $data['user_birthdate'] ) || $data['user_birthdate'] != '1900-01-01' || $data['user_birthdate'] != '1970-1-01' ) {
                    $dateinfo = explode( "-", $data['user_birthdate'] );
                    $text .= $data['user_name'] . ', ';
                    $info[$dateinfo[1] . '-' . $dateinfo[2]] = $text;
                }
            }
            $this->calinf['birthday'] = $info;
        }
    }

    public function get_Event($startd, $stopd) {
        $info = [];
        $result = dbquery( $this->sqlCal->getEventQuery( ['condition' => "(fh_o_start_date BETWEEN CAST( '" . $startd . "' AS DATE ) AND CAST( '" . $stopd . "' AS DATE ) ) OR (fh_o_start_ts BETWEEN CAST( '" . $startd . "' AS DATE ) AND CAST( '" . $stopd . "' AS DATE ) )", 'order' => "fh_ev_activ ASC, fh_o_start_ts ASC, fh_o_start_date ASC"] ) );
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $dateinfo = !empty($data['fh_evall_day']) ? $data['fh_o_start_date'] : $data['fh_o_start_ts'];
                if (preg_match( '/(\d+)[\.\/\-\ ](\d+)[\.\/\-\ ](\d+)/', $dateinfo, $date_matches ) ) {
                    $text = "-" . $data['fh_ev_subject'] . "\n -- " . $data['fh_ev_desc'] . "\n";
                    $info[$date_matches[2] . '-' . $date_matches[3]]['bginfo'][] = $text;
                    $info[$date_matches[2] . '-' . $date_matches[3]]['fhcolor'] = $data['fh_cat_color'];
                }
            }
            if ( !empty( $info ) ) {
                $text = "\n" . self::$locale['FCAL_020'] . "\n";
                foreach ( $info as $dayid => $fday ) {
                	$i = 0;
                    foreach ( $fday['bginfo'] as $fd ) {
                        if ( $i == 0) {
                            $inf[$dayid] = $text.$fd;
                        } else {
                            $inf[$dayid] .= $fd;
                        }
                        $i++;
                    }
                    $inf['fhcolor'][$dayid] = $info[$date_matches[2] . '-' . $date_matches[3]]['fhcolor'];
                }
            }
            $this->calinf['event'] = $inf;
        }
    }

    public function setDisplay() {
    	$info = [];
    	$firstday = self::setFirstDay( $this->fmonth, $this->fyear );
    	$lastday = self::setLastDay( $this->fmonth, $this->fyear );

        $enddays = self::allDay($firstday, $lastday );
        $weekend = self::fhDateWeek( $firstday );
        $weekend = $weekend < 10 ? str_replace('0', '', $weekend) : $weekend;
        $counter = 0;

        self::getWork_Day( $this->fyear, $this->fmonth );

        for ( $d = 0; $d < $enddays; $d++ ) {
            $fday = $firstday;
			$weekdate = date( 'Y-m-d', $fday +( $d * 86400 ) );
            $dateinfo = explode( "-", $weekdate );
            $linkmonth = $dateinfo[1] < 10 ? str_replace('0', '', $dateinfo[1]) : $dateinfo[1];
            $linkday = $dateinfo[2] < 10 ? str_replace('0', '', $dateinfo[2]) : $dateinfo[2];
            $bginfoDat = '';
            $mon_day = $dateinfo[1] . '-' . $dateinfo[2];
            $wdayClass = self::getDateDay( 'wday', $dateinfo[1], $dateinfo[2], $dateinfo[0] );

            $bday = !empty( $this->calinf['birthday'][$mon_day] ) ? $this->calinf['birthday'][$mon_day] : '';
            $even = !empty( $this->calinf['event'][$mon_day] ) ? $this->calinf['event'][$mon_day] : '';
            $holy = !empty( $this->calinf['holidays'][$mon_day] ) ? $this->calinf['holidays'][$mon_day] : '';

            $bginfoDat = $bday."\n".$even."\n".$holy;

            //class start
            $classDat = ( ( $wdayClass == 0 or $wdayClass == 6 ) ? "#ccc" : '' );
            if (!empty( $this->calinf['event']['fhcolor'][$mon_day] ) ) {
                //event
                $classDat = $this->calinf['event']['fhcolor'][$mon_day];
            } else if (!empty( $this->calinf['birthday'][$mon_day] ) ) {
                //szulinap
                $classDat = $this->category[1];
            } else if (!empty( $this->calinf['holidays'][$mon_day] ) ) {
                //unnepnap
                $classDat = $this->category[4];
            }
            //mai nap
            $classDat = !empty( $this->getToday( $dateinfo[1], $dateinfo[2] ) ) ? $this->getToday( $dateinfo[1], $dateinfo[2] ) : $classDat;
            $class = !empty( $classDat ) ? " style='background-color:" . $classDat . ";'" : '';
            //class end
            $bginfo = !empty( $bginfoDat ) ? " data-toggle='tooltip' title='" . $bginfoDat . "'" : '';

            $info[$dateinfo[0]][$dateinfo[1]][$dateinfo[2]] = [
                'class'    => $class,
                'midle'    => $bginfo,
                'weekend'  => $weekend,
                'daylink'  => "<a href='" . FCAL_PATH . "fh_events.php?year=" . $dateinfo[0] . "&month=" . $linkmonth . "&event_day=" . $linkday . "'>" . $linkday . "</a>"
            ];
            $counter++;
            ($counter % 7 == 0) ? $weekend++ : '';
        }
            //print_p($info);
        return $info;
    }

    public function getToday( $month, $today ) {
        if ( $this->fyear == date( 'Y' ) && $month == date( 'm' ) && $today == date( 'd' ) ) {
            return "#ff4136";
        }
        return;
    }

    public function setHeadLink() {
        $calmonth = self::calculateMonth();
        $info = [
            'left'   => [
                'link' => "href='" . FUSION_SELF . "?year=" . $calmonth['prev_year'] . "&month=" . $calmonth['prev_month'] . "'>",
                'icon' => "<i class='fa fa-caret-square-left' title='". self:: $locale['FCAL_021'] ."'></i>"
            ],
            'center' => [
                'link' => "href='".FCAL_PATH."fh_events.php?mcode=". $this->fmonth . "' title='" . self::$locale['FCAL_022'] . "'>". self::setCenterLink(),
                'icon' => ''
            ],
            'right'  => [
                'link' => "href='".FUSION_SELF."?year=". $calmonth['next_year'] ."&month=". $calmonth['next_month'] ."'>",
                'icon' => "<i class='fa fa-caret-square-right' title='". self::$locale['FCAL_023'] ."'></i>"
            ]
        ];
	    return $info;
    }

    public function setCenterLink() {
        $fullmonth = self::setFullMonths( $this->fmonth );

        switch ( $this->eventsettings['fh_date_format'] ) {
            case 0:
                $info = $fullmonth . " - " . $this->fyear;
                break;
            case 1:
                $info = $this->fyear . " - " . $fullmonth;
                break;
            case 2:
                $info = $fullmonth . " - " . $this->fyear;
                break;
            default:
                $info = $this->fyear . " - " . $fullmonth;
        }
        return $info;
    }

}
